@@ -27,12 +27,12 @@ import net.lingala.zip4j.model.LocalFileHeader;
 import net.lingala.zip4j.model.enums.AesVersion;
 import net.lingala.zip4j.model.enums.CompressionMethod;
 import net.lingala.zip4j.model.enums.EncryptionMethod;
-import net.lingala.zip4j.util.BitUtils;
 import net.lingala.zip4j.util.InternalZipConstants;
 
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.PushbackInputStream;
+import java.nio.charset.Charset;
 import java.util.List;
 import java.util.zip.CRC32;
 import java.util.zip.DataFormatException;
@@ -48,16 +48,31 @@ public class ZipInputStream extends InputStream {
   private char[] password;
   private LocalFileHeader localFileHeader;
   private CRC32 crc32 = new CRC32();
-  private boolean extraDataRecordReadForThisEntry = false;
   private byte[] endOfEntryBuffer;
+  private boolean extraDataRecordReadForThisEntry = false;
+  private boolean canSkipExtendedLocalFileHeader = false;
+  private Charset charset;
 
   public ZipInputStream(InputStream inputStream) {
-    this(inputStream, null);
+    this(inputStream, null, InternalZipConstants.CHARSET_UTF_8);
+  }
+
+  public ZipInputStream(InputStream inputStream, Charset charset) {
+    this(inputStream, null, charset);
   }
 
   public ZipInputStream(InputStream inputStream, char[] password) {
+    this(inputStream, password, InternalZipConstants.CHARSET_UTF_8);
+  }
+
+  public ZipInputStream(InputStream inputStream, char[] password, Charset charset) {
+    if(charset == null) {
+      charset = InternalZipConstants.CHARSET_UTF_8;
+    }
+
     this.inputStream = new PushbackInputStream(inputStream, 512);
     this.password = password;
+    this.charset = charset;
   }
 
   public LocalFileHeader getNextEntry() throws IOException {
@@ -69,7 +84,7 @@ public class ZipInputStream extends InputStream {
       readUntilEndOfEntry();
     }
 
-    localFileHeader = headerReader.readLocalFileHeader(inputStream);
+    localFileHeader = headerReader.readLocalFileHeader(inputStream, charset);
 
     if (localFileHeader == null) {
       return null;
@@ -82,6 +97,9 @@ public class ZipInputStream extends InputStream {
       localFileHeader.setCrc(fileHeader.getCrc());
       localFileHeader.setCompressedSize(fileHeader.getCompressedSize());
       localFileHeader.setUncompressedSize(fileHeader.getUncompressedSize());
+      canSkipExtendedLocalFileHeader = true;
+    } else {
+      canSkipExtendedLocalFileHeader = false;
     }
 
     if (!isZipEntryDirectory(localFileHeader.getFileName())) {
@@ -206,7 +224,7 @@ public class ZipInputStream extends InputStream {
   }
 
   private void readExtendedLocalFileHeaderIfPresent() throws IOException {
-    if (!isExtendedLocalFileHeaderPresent(localFileHeader)) {
+    if (!localFileHeader.isDataDescriptorExists() || canSkipExtendedLocalFileHeader) {
       return;
     }
 
@@ -217,11 +235,6 @@ public class ZipInputStream extends InputStream {
     localFileHeader.setCrc(dataDescriptor.getCrc());
   }
 
-  private boolean isExtendedLocalFileHeaderPresent(LocalFileHeader localFileHeader) {
-    byte[] generalPurposeFlags = localFileHeader.getGeneralPurposeFlag();
-    return BitUtils.isBitSet(generalPurposeFlags[0], 3);
-  }
-
   private void verifyLocalFileHeader(LocalFileHeader localFileHeader) throws IOException {
     if (!isEntryDirectory(localFileHeader.getFileName())
         && localFileHeader.getCompressionMethod() == CompressionMethod.STORE
@@ -253,7 +266,7 @@ public class ZipInputStream extends InputStream {
     }
 
     if (localFileHeader.getCrc() != crc32.getValue()) {
-      ZipException.Type exceptionType = ZipException.Type.UNKNOWN;
+      ZipException.Type exceptionType = ZipException.Type.CHECKSUM_MISMATCH;
 
       if (isEncryptionMethodZipStandard(localFileHeader)) {
         exceptionType = ZipException.Type.WRONG_PASSWORD;
@@ -278,7 +291,7 @@ public class ZipInputStream extends InputStream {
       return localFileHeader.getUncompressedSize();
     }
 
-    if (localFileHeader.isDataDescriptorExists()) {
+    if (localFileHeader.isDataDescriptorExists() && !canSkipExtendedLocalFileHeader) {
       return -1;
     }
 
@@ -315,5 +328,4 @@ public class ZipInputStream extends InputStream {
   private boolean isEncryptionMethodZipStandard(LocalFileHeader localFileHeader) {
     return localFileHeader.isEncrypted() && EncryptionMethod.ZIP_STANDARD.equals(localFileHeader.getEncryptionMethod());
   }
-
 }
